@@ -1,23 +1,47 @@
-
 import os
 import fnmatch
 import jinja2
 from collections import defaultdict
-import dbt.project
-from dbt.source import Source
-from dbt.utils import find_model_by_fqn, find_model_by_name, dependency_projects, split_path, This, Var, compiler_error
-from dbt.linker import Linker
-import dbt.targets
-import dbt.templates
 import time
 import sqlparse
 
-CompilableEntities = ["models", "tests", "archives", "analyses"]
+import dbt.project
+from dbt.source import Source
+from dbt.utils import find_model_by_fqn, find_model_by_name, \
+    dependency_projects, split_path, This, Var, compiler_error, \
+    to_string
+
+from dbt.linker import Linker
+from dbt.runtime import RuntimeContext
+import dbt.targets
+import dbt.templates
+
+from dbt.logger import GLOBAL_LOGGER as logger
+
+CompilableEntities = [
+    "models", "data tests", "schema tests", "archives", "analyses"
+]
+
+
+def compile_string(string, ctx):
+    try:
+        env = jinja2.Environment()
+        template = env.from_string(str(string), globals=ctx)
+        return template.render(ctx)
+    except jinja2.exceptions.TemplateSyntaxError as e:
+        compiler_error(None, str(e))
+    except jinja2.exceptions.UndefinedError as e:
+        compiler_error(None, str(e))
+
 
 class Compiler(object):
-    def __init__(self, project, create_template_class):
+    def __init__(self, project, create_template_class, args):
         self.project = project
         self.create_template = create_template_class()
+        self.args = args
+
+        self.project.args = args
+
         self.macro_generator = None
         self.target = self.get_target()
 
@@ -38,13 +62,23 @@ class Compiler(object):
 
         paths = own_project.get('source-paths', [])
         if self.create_template.label == 'build':
-            return Source(this_project, own_project=own_project).get_models(paths, self.create_template)
+            return Source(
+                this_project,
+                own_project=own_project
+            ).get_models(paths, self.create_template)
+
         elif self.create_template.label == 'test':
-            return Source(this_project, own_project=own_project).get_test_models(paths, self.create_template)
+            return Source(
+                this_project,
+                own_project=own_project
+            ).get_test_models(paths, self.create_template)
+
         elif self.create_template.label == 'archive':
             return []
         else:
-            raise RuntimeError("unexpected create template type: '{}'".format(self.create_template.label))
+            raise RuntimeError(
+                "unexpected create template "
+                "type: '{}'".format(self.create_template.label))
 
     def get_macros(self, this_project, own_project=None):
         if own_project is None:
@@ -54,12 +88,19 @@ class Compiler(object):
 
     def get_archives(self, project):
         archive_template = dbt.templates.ArchiveInsertTemplate()
-        return Source(project, own_project=project).get_archives(archive_template)
+        return Source(
+            project,
+            own_project=project
+        ).get_archives(archive_template)
 
     def project_schemas(self):
         source_paths = self.project.get('source-paths', [])
         return Source(self.project).get_schemas(source_paths)
 
+    def project_tests(self):
+        source_paths = self.project.get('test-paths', [])
+        return Source(self.project).get_tests(source_paths)
+
     def analysis_sources(self, project):
         paths = project.get('analysis-paths', [])
         return Source(project).get_analyses(paths)
@@ -70,8 +111,15 @@ class Compiler(object):
             found_models[model.name].append(model)
         for model_name, model_list in found_models.items():
             if len(model_list) > 1:
-                models_str = "\n  - ".join([str(model) for model in model_list])
-                raise RuntimeError("Found {} models with the same name! Can't create tables. Name='{}'\n  - {}".format(len(model_list), model_name, models_str))
+                models_str = "\n  - ".join(
+                    [str(model) for model in model_list])
+
+                raise RuntimeError(
+                    "Found {} models with the same name! Can't "
+                    "create tables. Name='{}'\n  - {}".format(
+                        len(model_list), model_name, models_str
+                    )
+                )
 
     def __write(self, build_filepath, payload):
         target_path = os.path.join(self.project['target-path'], build_filepath)
@@ -80,8 +128,7 @@ class Compiler(object):
             os.makedirs(os.path.dirname(target_path))
 
         with open(target_path, 'w') as f:
-            f.write(payload)
-
+            f.write(to_string(payload))
 
     def __model_config(self, model, linker):
         def do_config(*args, **kwargs):
@@ -90,10 +137,14 @@ class Compiler(object):
             elif len(args) == 0 and len(kwargs) > 0:
                 opts = kwargs
             else:
-                raise RuntimeError("Invalid model config given inline in {}".format(model))
+                raise RuntimeError(
+                    "Invalid model config given inline in {}".format(model)
+                )
 
             if type(opts) != dict:
-                raise RuntimeError("Invalid model config given inline in {}".format(model))
+                raise RuntimeError(
+                    "Invalid model config given inline in {}".format(model)
+                )
 
             model.update_in_model_config(opts)
             model.add_to_prologue("Config specified in model: {}".format(opts))
@@ -101,15 +152,19 @@ class Compiler(object):
         return do_config
 
     def model_can_reference(self, src_model, other_model):
-        """returns True if the src_model can reference the other_model. Models can access
-        other models in their package and dependency models, but a dependency model cannot
-        access models "up" the dependency chain"""
+        """
+        returns True if the src_model can reference the other_model. Models
+        can access other models in their package and dependency models, but
+        a dependency model cannot access models "up" the dependency chain.
+        """
 
         # hack for now b/c we don't support recursive dependencies
-        return other_model.own_project['name'] == src_model.own_project['name'] \
-                or src_model.own_project['name'] == src_model.project['name']
+        return (
+            other_model.own_project['name'] == src_model.own_project['name'] or
+            src_model.own_project['name'] == src_model.project['name']
+        )
 
-    def __ref(self, linker, ctx, model, all_models):
+    def __ref(self, linker, ctx, model, all_models, add_dependency=True):
         schema = ctx['env']['schema']
 
         source_model = tuple(model.fqn)
@@ -121,24 +176,36 @@ class Compiler(object):
                 other_model = find_model_by_name(all_models, other_model_name)
             elif len(args) == 2:
                 other_model_package, other_model_name = args
-                other_model_name = self.create_template.model_name(other_model_name)
-                other_model = find_model_by_name(all_models, other_model_name, package_namespace=other_model_package)
+                other_model_name = self.create_template.model_name(
+                    other_model_name
+                )
+
+                other_model = find_model_by_name(
+                    all_models,
+                    other_model_name,
+                    package_namespace=other_model_package
+                )
             else:
-                compiler_error(model, "ref() takes at most two arguments ({} given)".format(len(args)))
+                compiler_error(
+                    model,
+                    "ref() takes at most two arguments ({} given)".format(
+                        len(args)
+                    )
+                )
 
             other_model_fqn = tuple(other_model.fqn[:-1] + [other_model_name])
             src_fqn = ".".join(source_model)
             ref_fqn = ".".join(other_model_fqn)
 
-            #if not self.model_can_reference(model, other_model):
-            #    compiler_error(model, "Model '{}' exists but cannot be referenced from dependency model '{}'".format(ref_fqn, src_fqn))
-
             if not other_model.is_enabled:
-                raise RuntimeError("Model '{}' depends on model '{}' which is disabled in the project config".format(src_fqn, ref_fqn))
+                raise RuntimeError(
+                    "Model '{}' depends on model '{}' which is disabled in "
+                    "the project config".format(src_fqn, ref_fqn)
+                )
 
             # this creates a trivial cycle -- should this be a compiler error?
             # we can still interpolate the name w/o making a self-cycle
-            if source_model == other_model_fqn:
+            if source_model == other_model_fqn or not add_dependency:
                 pass
             else:
                 linker.dependency(source_model, other_model_fqn)
@@ -153,51 +220,76 @@ class Compiler(object):
             try:
                 return do_ref(*args)
             except RuntimeError as e:
-                root = os.path.relpath(model.root_dir, model.project['project-root'])
+                root = os.path.relpath(
+                    model.root_dir,
+                    model.project['project-root']
+                )
+
                 filepath = os.path.join(root, model.rel_filepath)
-                print("Compiler error in {}".format(filepath))
-                print("Enabled models:")
+                logger.info("Compiler error in {}".format(filepath))
+                logger.info("Enabled models:")
                 for m in all_models:
-                    print(" - {}".format(".".join(m.fqn)))
+                    logger.info(" - {}".format(".".join(m.fqn)))
                 raise e
 
         return wrapped_do_ref
 
-    def get_context(self, linker, model,  models):
+    def get_context(self, linker, model,  models, add_dependency=False):
+        runtime = RuntimeContext(model=model)
+
         context = self.project.context()
 
         # built-ins
-        context['ref'] = self.__ref(linker, context, model, models)
+        context['ref'] = self.__ref(
+            linker, context, model, models, add_dependency
+        )
         context['config'] = self.__model_config(model, linker)
-        context['this'] = This(context['env']['schema'], model.immediate_name, model.name)
+        context['this'] = This(
+            context['env']['schema'], model.immediate_name, model.name
+        )
         context['var'] = Var(model, context=context)
+        context['target'] = self.project.get_target()
 
         # these get re-interpolated at runtime!
         context['run_started_at'] = '{{ run_started_at }}'
-        context['invocation_id']  = '{{ invocation_id }}'
+        context['invocation_id'] = '{{ invocation_id }}'
 
         # add in context from run target
         context.update(self.target.context)
 
+        runtime.update_global(context)
+
         # add in macros (can we cache these somehow?)
-        for macro_name, macro in self.macro_generator(context):
-            context[macro_name] = macro
+        for macro_data in self.macro_generator(context):
+            macro = macro_data["macro"]
+            macro_name = macro_data["name"]
+            project = macro_data["project"]
 
-        return context
+            runtime.update_package(project['name'], {macro_name: macro})
 
-    def compile_model(self, linker, model, models):
+            if project['name'] == self.project['name']:
+                runtime.update_global({macro_name: macro})
+
+        return runtime
+
+    def compile_model(self, linker, model, models, add_dependency=True):
         try:
             fs_loader = jinja2.FileSystemLoader(searchpath=model.root_dir)
             jinja = jinja2.Environment(loader=fs_loader)
 
-            # this is a dumb jinja2 bug -- on windows, forward slashes are EXPECTED
+            # this is a dumb jinja2 bug -- on windows, forward slashes
+            # are EXPECTED
             posix_filepath = '/'.join(split_path(model.rel_filepath))
             template = jinja.get_template(posix_filepath)
-            context = self.get_context(linker, model, models)
+            context = self.get_context(
+                linker, model, models, add_dependency=add_dependency
+            )
 
             rendered = template.render(context)
         except jinja2.exceptions.TemplateSyntaxError as e:
             compiler_error(model, str(e))
+        except jinja2.exceptions.UndefinedError as e:
+            compiler_error(model, str(e))
 
         return rendered
 
@@ -209,7 +301,11 @@ class Compiler(object):
     def combine_query_with_ctes(self, model, query, ctes, compiled_models):
         parsed_stmts = sqlparse.parse(query)
         if len(parsed_stmts) != 1:
-            raise RuntimeError("unexpectedly parsed {} queries from model {}".format(len(parsed_stmts), model))
+            raise RuntimeError(
+                "unexpectedly parsed {} queries from model "
+                "{}".format(len(parsed_stmts), model)
+            )
+
         parsed = parsed_stmts[0]
 
         with_stmt = None
@@ -224,16 +320,27 @@ class Compiler(object):
             with_stmt = sqlparse.sql.Token(sqlparse.tokens.Keyword, 'with')
             parsed.insert_before(first_token, with_stmt)
         else:
-            # stmt exists, add a comma (which will come after our injected CTE(s) )
-            trailing_comma = sqlparse.sql.Token(sqlparse.tokens.Punctuation, ',')
+            # stmt exists, add a comma (which will come after our injected
+            # CTE(s) )
+            trailing_comma = sqlparse.sql.Token(
+                sqlparse.tokens.Punctuation, ','
+            )
             parsed.insert_after(with_stmt, trailing_comma)
 
-        cte_mapping = [(model.cte_name, compiled_models[model]) for model in ctes]
+        cte_mapping = [
+            (model.cte_name, compiled_models[model]) for model in ctes
+        ]
 
-        # these newlines are important -- comments could otherwise interfere w/ query
-        cte_stmts = [" {} as (\n{}\n)".format(name, contents) for (name, contents) in cte_mapping]
+        # these newlines are important -- comments could otherwise interfere
+        # w/ query
+        cte_stmts = [
+            " {} as (\n{}\n)".format(name, contents)
+            for (name, contents) in cte_mapping
+        ]
 
-        cte_text = sqlparse.sql.Token(sqlparse.tokens.Keyword, ", ".join(cte_stmts))
+        cte_text = sqlparse.sql.Token(
+            sqlparse.tokens.Keyword, ", ".join(cte_stmts)
+        )
         parsed.insert_after(with_stmt, cte_text)
 
         return str(parsed)
@@ -243,12 +350,18 @@ class Compiler(object):
             return set()
 
         models_to_add = linker.cte_map[model]
-        recursive_models = [self.__recursive_add_ctes(linker, m) for m in models_to_add]
+        recursive_models = [
+            self.__recursive_add_ctes(linker, m) for m in models_to_add
+        ]
+
         for recursive_model_set in recursive_models:
-             models_to_add = models_to_add | recursive_model_set
+            models_to_add = models_to_add | recursive_model_set
+
         return models_to_add
 
-    def add_cte_to_rendered_query(self, linker, primary_model, compiled_models):
+    def add_cte_to_rendered_query(
+            self, linker, primary_model, compiled_models
+    ):
         fqn_to_model = {tuple(model.fqn): model for model in compiled_models}
         sorted_nodes = linker.as_topological_ordering()
 
@@ -261,7 +374,7 @@ class Compiler(object):
                 continue
 
             model = fqn_to_model[node]
-            # add these in topological sort order -- that's significant for CTEs
+            # add these in topological sort order -- significant for CTEs
             if model.is_ephemeral and model in models_to_add:
                 required_ctes.append(model)
 
@@ -269,18 +382,50 @@ class Compiler(object):
         if len(required_ctes) == 0:
             return query
         else:
-            compiled_query = self.combine_query_with_ctes(primary_model, query, required_ctes, compiled_models)
+            compiled_query = self.combine_query_with_ctes(
+                primary_model, query, required_ctes, compiled_models
+            )
             return compiled_query
 
+    def remove_node_from_graph(self, linker, model, models):
+        # remove the node
+        children = linker.remove_node(tuple(model.fqn))
+
+        # check if we bricked the graph. if so: throw compilation error
+        for child in children:
+            other_model = find_model_by_fqn(models, child)
+
+            if other_model.is_enabled:
+                this_fqn = ".".join(model.fqn)
+                that_fqn = ".".join(other_model.fqn)
+                compiler_error(
+                    model,
+                    "Model '{}' depends on model '{}' which is "
+                    "disabled".format(that_fqn, this_fqn)
+                )
+
     def compile_models(self, linker, models):
-        compiled_models = {model: self.compile_model(linker, model, models) for model in models}
-        sorted_models = [find_model_by_fqn(models, fqn) for fqn in linker.as_topological_ordering()]
+        compiled_models = {model: self.compile_model(linker, model, models)
+                           for model in models}
+        sorted_models = [find_model_by_fqn(models, fqn)
+                         for fqn in linker.as_topological_ordering()]
 
         written_models = []
         for model in sorted_models:
-            injected_stmt = self.add_cte_to_rendered_query(linker, model, compiled_models)
+            # in-model configs were just evaluated. Evict anything that is
+            # newly-disabled
+            if not model.is_enabled:
+                self.remove_node_from_graph(linker, model, models)
+                continue
+
+            injected_stmt = self.add_cte_to_rendered_query(
+                linker, model, compiled_models
+            )
+
             context = self.get_context(linker, model, models)
-            wrapped_stmt = model.compile(injected_stmt, self.project, self.create_template, context)
+            wrapped_stmt = model.compile(
+                injected_stmt, self.project, self.create_template, context
+            )
 
             serialized = model.serialize()
             linker.update_node_data(tuple(model.fqn), serialized)
@@ -295,14 +440,22 @@ class Compiler(object):
 
     def compile_analyses(self, linker, compiled_models):
         analyses = self.analysis_sources(self.project)
-        compiled_analyses = {analysis: self.compile_model(linker, analysis, compiled_models) for analysis in analyses}
+        compiled_analyses = {
+            analysis: self.compile_model(
+                linker, analysis, compiled_models
+            ) for analysis in analyses
+        }
 
         written_analyses = []
         referenceable_models = {}
         referenceable_models.update(compiled_models)
         referenceable_models.update(compiled_analyses)
         for analysis in analyses:
-            injected_stmt = self.add_cte_to_rendered_query(linker, analysis, referenceable_models)
+            injected_stmt = self.add_cte_to_rendered_query(
+                linker,
+                analysis,
+                referenceable_models
+            )
             build_path = analysis.build_path()
             self.__write(build_path, injected_stmt)
             written_analyses.append(analysis)
@@ -316,7 +469,8 @@ class Compiler(object):
 
         schema_tests = []
         for schema in schemas:
-            schema_tests.extend(schema.compile()) # compiling a SchemaFile returns >= 0 SchemaTest models
+            # compiling a SchemaFile returns >= 0 SchemaTest models
+            schema_tests.extend(schema.compile())
 
         written_tests = []
         for schema_test in schema_tests:
@@ -329,6 +483,25 @@ class Compiler(object):
 
         return written_tests
 
+    def compile_data_tests(self, linker):
+        tests = self.project_tests()
+
+        all_models = self.get_models()
+        enabled_models = [model for model in all_models if model.is_enabled]
+
+        written_tests = []
+        for data_test in tests:
+            serialized = data_test.serialize()
+            linker.update_node_data(tuple(data_test.fqn), serialized)
+            query = self.compile_model(
+                linker, data_test, enabled_models, add_dependency=False
+            )
+            wrapped = data_test.render(query)
+            self.__write(data_test.build_path(), wrapped)
+            written_tests.append(data_test)
+
+        return written_tests
+
     def generate_macros(self, all_macros):
         def do_gen(ctx):
             macros = []
@@ -351,40 +524,69 @@ class Compiler(object):
         self.write_graph_file(linker, 'archive')
         return all_archives
 
-    def compile(self, dry=False):
+    def get_models(self):
+        all_models = self.model_sources(this_project=self.project)
+        for project in dependency_projects(self.project):
+            all_models.extend(
+                self.model_sources(
+                    this_project=self.project, own_project=project
+                )
+            )
+
+        return all_models
+
+    def compile(self, limit_to=None):
         linker = Linker()
 
-        all_models = self.model_sources(this_project=self.project)
+        all_models = self.get_models()
         all_macros = self.get_macros(this_project=self.project)
 
         for project in dependency_projects(self.project):
-            all_models.extend(self.model_sources(this_project=self.project, own_project=project))
-            all_macros.extend(self.get_macros(this_project=self.project, own_project=project))
+            all_macros.extend(
+                self.get_macros(this_project=self.project, own_project=project)
+            )
 
         self.macro_generator = self.generate_macros(all_macros)
 
-        enabled_models = [model for model in all_models if model.is_enabled]
+        if limit_to is not None and 'models' in limit_to:
+            enabled_models = [
+                model for model in all_models
+                if model.is_enabled and not model.is_empty
+            ]
+        else:
+            enabled_models = []
 
-        compiled_models, written_models = self.compile_models(linker, enabled_models)
+        compiled_models, written_models = self.compile_models(
+            linker, enabled_models
+        )
 
         # TODO : only compile schema tests for enabled models
-        written_schema_tests = self.compile_schema_tests(linker)
+        if limit_to is not None and 'tests' in limit_to:
+            written_schema_tests = self.compile_schema_tests(linker)
+            written_data_tests = self.compile_data_tests(linker)
+        else:
+            written_schema_tests = []
+            written_data_tests = []
 
         self.validate_models_unique(compiled_models)
         self.validate_models_unique(written_schema_tests)
         self.write_graph_file(linker, self.create_template.label)
 
-        if self.create_template.label not in ['test', 'archive']:
+        if limit_to is not None and 'analyses' in limit_to and \
+           self.create_template.label not in ['test', 'archive']:
             written_analyses = self.compile_analyses(linker, compiled_models)
         else:
             written_analyses = []
 
-
-        compiled_archives = self.compile_archives()
+        if limit_to is not None and 'archives' in limit_to:
+            compiled_archives = self.compile_archives()
+        else:
+            compiled_archives = []
 
         return {
             "models": len(written_models),
-            "tests" : len(written_schema_tests),
+            "schema tests": len(written_schema_tests),
+            "data tests": len(written_data_tests),
             "archives": len(compiled_archives),
-            "analyses" : len(written_analyses)
+            "analyses": len(written_analyses)
         }
