@@ -17,10 +17,12 @@ class CleanTask:
 
     def __is_protected_path(self, path):
         abs_path = os.path.abspath(path)
-        protected_paths = self.project['source-paths'] + self.project['test-paths'] + ['.']
+        protected_paths = self.project['source-paths'] + \
+            self.project['test-paths'] + ['.']
 
         protected_abs_paths = [os.path.abspath for p in protected_paths]
-        return abs_path in set(protected_abs_paths) or self.__is_project_path(abs_path)
+        return abs_path in set(protected_abs_paths) or \
+            self.__is_project_path(abs_path)
 
     def run(self):
         for path in self.project['clean-targets']:
