@@ -1,6 +1,6 @@
-
 from dbt.compilation import Compiler, CompilableEntities
 from dbt.templates import BaseCreateTemplate, DryCreateTemplate
+from dbt.logger import GLOBAL_LOGGER as logger
 
 
 class CompileTask:
@@ -14,9 +14,11 @@ class CompileTask:
         else:
             create_template = BaseCreateTemplate
 
-        compiler = Compiler(self.project, create_template)
+        compiler = Compiler(self.project, create_template, self.args)
         compiler.initialize()
-        results = compiler.compile(dry=self.args.dry)
+        results = compiler.compile(limit_to=CompilableEntities)
 
-        stat_line = ", ".join(["{} {}".format(results[k], k) for k in CompilableEntities])
-        print("Compiled {}".format(stat_line))
+        stat_line = ", ".join(
+            ["{} {}".format(results[k], k) for k in CompilableEntities]
+        )
+        logger.info("Compiled {}".format(stat_line))
