@@ -6,18 +6,22 @@ import pprint
 import subprocess
 import dbt.project as project
 
+from dbt.logger import GLOBAL_LOGGER as logger
+
+
 def folder_from_git_remote(remote_spec):
     start = remote_spec.rfind('/') + 1
     end = len(remote_spec) - (4 if remote_spec.endswith('.git') else 0)
     return remote_spec[start:end]
 
+
 class DepsTask:
     def __init__(self, args, project):
         self.args = args
         self.project = project
 
     def __checkout_branch(self, branch, full_path):
-        print("  checking out branch {}".format(branch))
+        logger.info("  checking out branch {}".format(branch))
         proc = subprocess.Popen(
             ['git', 'checkout', branch],
             cwd=full_path,
@@ -34,11 +38,15 @@ class DepsTask:
 
         out, err = proc.communicate()
 
-        exists = re.match("fatal: destination path '(.+)' already exists", err.decode('utf-8'))
+        exists = re.match(
+            "fatal: destination path '(.+)' already exists",
+            err.decode('utf-8')
+        )
+
         folder = None
         if exists:
             folder = exists.group(1)
-            print("updating existing dependency {}".format(folder))
+            logger.info("updating existing dependency {}".format(folder))
             full_path = os.path.join(self.project['modules-path'], folder)
             proc = subprocess.Popen(
                 ['git', 'fetch', '--all'],
@@ -46,7 +54,8 @@ class DepsTask:
                 stdout=subprocess.PIPE,
                 stderr=subprocess.PIPE)
             out, err = proc.communicate()
-            remote_branch = 'origin/master' if branch is None else 'origin/{}'.format(branch)
+            remote_branch = 'origin/master' if branch is None \
+                            else 'origin/{}'.format(branch)
             proc = subprocess.Popen(
                 ['git', 'reset', '--hard', remote_branch],
                 cwd=full_path,
@@ -59,7 +68,7 @@ class DepsTask:
             matches = re.match("Cloning into '(.+)'", err.decode('utf-8'))
             folder = matches.group(1)
             full_path = os.path.join(self.project['modules-path'], folder)
-            print("pulled new dependency {}".format(folder))
+            logger.info("pulled new dependency {}".format(folder))
             if branch is not None:
                 self.__checkout_branch(branch, full_path)
 
@@ -67,7 +76,11 @@ class DepsTask:
 
     def __split_at_branch(self, repo_spec):
         parts = repo_spec.split("@")
-        error = RuntimeError("Invalid dep specified: '{}' -- not a repo we can clone".format(repo_spec))
+        error = RuntimeError(
+            "Invalid dep specified: '{}' -- not a repo we can clone".format(
+                repo_spec
+            )
+        )
 
         repo = None
         if repo_spec.startswith("git@"):
@@ -88,26 +101,36 @@ class DepsTask:
 
         return repo, branch
 
-    def __pull_deps_recursive(self, repos, processed_repos = set(), i=0):
+    def __pull_deps_recursive(self, repos, processed_repos=None, i=0):
+        if processed_repos is None:
+            processed_repos = set()
         for repo_string in repos:
             repo, branch = self.__split_at_branch(repo_string)
             repo_folder = folder_from_git_remote(repo)
 
             try:
                 if repo_folder in processed_repos:
-                    print("skipping already processed dependency {}".format(repo_folder))
+                    logger.info(
+                        "skipping already processed dependency {}"
+                        .format(repo_folder)
+                    )
                 else:
                     dep_folder = self.__pull_repo(repo, branch)
                     dep_project = project.read_project(
                         os.path.join(self.project['modules-path'],
                                      dep_folder,
-                                     'dbt_project.yml'), profile_to_load=self.project.profile_to_load
+                                     'dbt_project.yml'),
+                        self.project.profiles_dir,
+                        profile_to_load=self.project.profile_to_load
                     )
                     processed_repos.add(dep_folder)
-                    self.__pull_deps_recursive(dep_project['repositories'], processed_repos, i+1)
+                    self.__pull_deps_recursive(
+                        dep_project['repositories'], processed_repos, i+1
+                    )
             except IOError as e:
                 if e.errno == errno.ENOENT:
-                    print("'{}' is not a valid dbt project - dbt_project.yml not found".format(repo))
+                    logger.info("'{}' is not a valid dbt project - "
+                                "dbt_project.yml not found".format(repo))
                     exit(1)
                 else:
                     raise e
